﻿//
// リモート電圧制御電源テストプログラム
//
var http = require("http");
var url  = require("url");
var exec = require("child_process").exec;

//
// グラフ表示用バッファ関係
//
var BUFSIZE = 101;
var datp = 0;
var data = new Array(BUFSIZE);

//
//　出力電圧・継続時間リスト（プログラム）関係
//
var CTRL_SIZE = 8;
var ctrl_v = new Array(CTRL_SIZE);
var ctrl_t = new Array(CTRL_SIZE);
var cur_step = 0;	// Output Step Number
var cur_t = 0;		// Current Time
var dacdat = 0;		// Output Voltage

//
//　start！
//
function start(route, handle) {

//
// 配列等の初期化
//
function init_data() {
	var i;
	var d;
	for (i=0; i<BUFSIZE; i++) {
		d = Math.sin(Math.PI/50.0*i);
		data[i] = ~~(d*100+128);
	}
	for (i=0; i<CTRL_SIZE; i++) {
		ctrl_v[i] = 0;
		ctrl_t[i] = 0;
	}
	var cmd = "./dacout ";
	exec(
		cmd,
		function(err, stdout, stderr) {}
	);
}

//
// DAC出力
//
function dac_out() {
	dacdat = ctrl_v[cur_step];
	cur_t++;
	if (cur_t >= ctrl_t[cur_step]) {
		cur_t = 0;
		cur_step++;
		if (cur_step >= CTRL_SIZE) {
			cur_step = 0;
		}
	}
//	var cmd = "./i2cdummy "+dacdat.toString(16);
	var cmd = "./dacout "+ dacdat.toString(16);
//	var cmd = 'i2cget -y 1 0x08 0x00';
//	console.log(cmd);

	exec(
		cmd,
		function(err, stdout, stderr) {
			data[datp] = parseInt(stdout,16) * 16;
//			console.log('=>'+data[datp]);
			datp++;
			if (datp >= BUFSIZE) {
				datp = 0;
			}
		}
	);
}

//
//　グラフやセレクトボックス表示用のスクリプト作成
//
function draw_graph(ref) {
	var i,j;
	var selbox;

	// 電圧セレクトボックスの作成
	selbox = '<TT>Volt</TT>';
	for (j=0; j<8; j++) {
		selbox = selbox + '\t' + '<select name = "V' + j.toString(10) + '" id = "v' + j.toString(10) + '" value = "' + j.toString(10) + '">' + '\n';
		for (i=0; i<16; i++) {
			selbox = selbox + '\t\t' + '<option';
			if (i == ctrl_v[j]) {
				selbox = selbox + ' selected';
			}
			selbox = selbox + ' value = "' + i.toString(10) + '">' + ((i*0.2).toFixed(1)).toString(10) + '</option>' + '\n';
		}
		selbox = selbox + '\t' + '</select>' + '\n';
	}
	selbox = selbox + '\t<BR>\n';

	// 継続時間セレクトボックスの作成
	selbox = selbox + '<TT>Time</TT>';
	for (j=0; j<8; j++) {
		selbox = selbox + '\t' + '<select name = "T' + j.toString(10) + '" id = "t' + j.toString(10) + '" value = "' + j.toString(10) + '">' + '\n';
		for (i=1; i<10; i++) {
			selbox = selbox + '\t\t' + '<option';
			if (i == ctrl_t[j]) {
				selbox = selbox + ' selected';
			}
			selbox = selbox + ' value = "' + i.toString(10) + '">' + (i.toFixed(1)).toString(10) + '</option>' + '\n';
		}
		selbox = selbox + '\t' + '</select>' + '\n';
	}
	selbox = selbox + '\t<BR>\n'

	// グラフ用データリスト作成
	var valbuf = '\tdata[0] = ' +data[0].toString(10) + ';\n';
	for (i=1; i<BUFSIZE; i++) {
	  valbuf = valbuf+ '\t'+
	    'data['+ i + '] = ' + data[i].toString(10) + ';\n';
	}
//	console.log(valbuf);

	// グラフ描画用スクリプト
	var drawbuf = '<!DOCTYPE html>' + '\n'+
	'<html>' + '\n'+
	'<head>' + '\n'+
	'<meta charset="utf-8">' + '\n'+
	'<title>Remote Power Supply</title>' + '\n'+
	'<style type = "text/css">' + '\n'+
	'canvas {border: 1px solid black; }' + '\n'+
	'</style>' + '\n'+
	'<script type = "text/javascript">' + '\n'+
	'const CHEIGHT = 300;' + '\n' +
	'const CWIDTH  = 400;' + '\n' +
	'const BUFSIZE = 101;' + '\n' +
	'var data = Array(BUFSIZE); ' + '\n' +
	'var datp = ' + datp + ';' + '\n' +

	valbuf + 

	'function draw(context) {' + '\n'+
	'	var i;' + '\n' + 
	'	context.fillStyle=\'rgb(255,0,0)\';' + '\n'+
	'	context.strokeStyle = "black";' + '\n'+
	'	context.lineWidth = 1;' + '\n'+
	'	context.lineCap = \'round\';' + '\n'+
	'	context.beginPath();' + '\n'+
	'	context.moveTo(0, CHEIGHT/2);' + '\n' + 
	'	context.lineTo(CWIDTH, CHEIGHT/2);' + '\n' + 
	'	for (i=0; i<BUFSIZE; i+=10) {' + '\n'+
	'		context.moveTo(i*4, (CHEIGHT/2)-10);'+'\n'+
	'		context.lineTo(i*4, (CHEIGHT/2)+10);'+'\n'+
	'	}' + '\n' +
	'	context.moveTo(0, 128- data[datp++] + CHEIGHT/2);' + '\n' +
	'	for (i=1; i<BUFSIZE; i++) {' + '\n' +
	'	  if (datp >= BUFSIZE) datp = 0;' + '\n' +
	'	  context.lineTo(i*4, 128 - data[datp++] + CHEIGHT/2);'+'\n' + 
	'	}' + '\n' +
	'	context.stroke();' + '\n'+
	'}' + '\n'+
	'window.onload = function() {' + '\n'+
	'	var canvas = document.getElementById(\'mycanvas\');' + '\n'+
	'	if (!canvas.getContext) {' + '\n'+
	'		return;' + '\n'+
	'	}' + '\n'+
	'	var context = canvas.getContext(\'2d\');' + '\n'+
	'	draw(context);' + '\n'+
	'}' + '\n'+
	'</script>' + '\n' ; 

	if (ref != 0) {	// オートリフレッシュモードだったら、3秒後に/refresh1.htmlに飛ぶ
		drawbuf = drawbuf + '<meta http-equiv="refresh" content="3;URL = /refresh'+ref.toString(10)+'.html">' + '\n'; 
	}

	drawbuf = drawbuf + 
	'</head>' + '\n'+
	'<body>' + '\n'+
	'        <canvas id = "mycanvas" width="400" height="300">' + '\n'+
	'		Canvas not supported.' + '\n'+
	'        </canvas>' + '\n';

	if (ref != 0) {	// オートリフレッシュモードか否かが分かるようにテキストで表示
		drawbuf = drawbuf + '<BR><TT><I>Auto Refresh</I>:ON </TT><BR>' + '\n';
	} else {
		drawbuf = drawbuf + '<BR><TT><I>Auto Refresh</I>:OFF</TT><BR>' + '\n';
	}

	drawbuf = drawbuf + 
	'	<form action="/set.html" method="post">' + '\n' +
	'	<br>' + '\n' + 
	selbox + 
  	'	<input type="submit" value="Set">' + '\n' +
	'	</form>' + '\n';
	if (ref != 0) {
		drawbuf = drawbuf + '	<form action="/" method="post">' + '\n' ;
	} else {
		drawbuf = drawbuf + '	<form action="/refresh1.html" method="post">' + '\n' ;
	}

	drawbuf = drawbuf + 
  	'	<input type="submit" value="Refresh">' + '\n' +
//	'	<input type="checkbox" name="chk1" checked/><BR>' + '\n' +
//	'	<br>' + '\n' + 
	'	</form>' + '\n' +


	'</body>' + '\n'+
	'</html>'+'\n';
    return(drawbuf);
}

//
//　表示データ作成
//
function onRequest(req, res) {
		var pathname = url.parse(req.url).pathname;
		var query = url.parse(req.url).query;
		var dat;
		var auto_refresh;
		switch(pathname) {
			case "/refresh1.html":	auto_refresh = 2; break;
			case "/refresh2.html":	auto_refresh = 1; break;
			case "/set.html":	auto_refresh = 1; break;
			default: auto_refresh = 0; break;
		}
		console.log("Request for "+ pathname + " received.");
		console.log("Query       "+ query + " received.");

		req.setEncoding("utf8");
		if (pathname =="/set.html") {
		    req.addListener("data", function(postDataChunk) {
			console.log("Received POST data chunk '" + postDataChunk + "'.");
			var postlist = (new String(postDataChunk)).split('&');
			for (var postkey in postlist) {
				var postcol = (postlist[postkey]).split('=');
//				console.log("  ["+postkey + "]  "+ postcol[0] + "=" + postcol[1]);
				dat = parseInt(postcol[1]);
				switch (postcol[0]) {
					case "V0": ctrl_v[0] = dat; cur_step = 0; cur_t = 0;break;
					case "V1": ctrl_v[1] = dat; break;
					case "V2": ctrl_v[2] = dat; break;
					case "V3": ctrl_v[3] = dat; break;
					case "V4": ctrl_v[4] = dat; break;
					case "V5": ctrl_v[5] = dat; break;
					case "V6": ctrl_v[6] = dat; break;
					case "V7": ctrl_v[7] = dat; break;
					case "T0": ctrl_t[0] = dat; break;
					case "T1": ctrl_t[1] = dat; break;
					case "T2": ctrl_t[2] = dat; break;
					case "T3": ctrl_t[3] = dat; break;
					case "T4": ctrl_t[4] = dat; break;
					case "T5": ctrl_t[5] = dat; break;
					case "T6": ctrl_t[6] = dat; break;
					case "T7": ctrl_t[7] = dat; break;
					default: break;
				}
			}
			var i = 0;
			for (i=0; i<7; i++) {
				console.log( "V" + i + "=" + ctrl_v[i] + " T" + i + "=" +ctrl_t[i]);
			}
			var drawbuf = draw_graph(auto_refresh);
			res.writeHead(200, { "Content-Type":"text/html"});
			res.write(drawbuf);
			res.end();
		    });
		} else {
			var drawbuf = draw_graph(auto_refresh);
			res.writeHead(200, { "Content-Type":"text/html"});
			res.write(drawbuf);
			res.end();
		}
}

function dacout() {
	setInterval(function() {
		dac_out();
	}, 1000);
}

init_data();
http.createServer( onRequest ).listen(8000);
console.log("Waiting for Port 8000");
dacout();

}	// end of 'start('

start();
